@@ -469,6 +469,11 @@ gst_gl_window_wayland_egl_close (GstGLWindow * gl_window)
 
   destroy_surfaces (window_egl);
 
+  /* As we are about to destroy the wl_source, we need to ensure everything
+   * has been sent synchronously, otherwise we will be leaking surfaces on
+   * server, leaving the window visible and unrefreshed on screen. */
+  wl_display_flush (GST_GL_DISPLAY_WAYLAND (gl_window->display)->display);
+
   g_source_destroy (window_egl->wl_source);
   g_source_unref (window_egl->wl_source);
   window_egl->wl_source = NULL;
@@ -494,7 +499,8 @@ registry_handle_global (void *data, struct wl_registry *registry,
   GstGLWindowWaylandEGL *window_wayland = data;
 
   GST_TRACE_OBJECT (window_wayland, "registry_handle_global with registry %p, "
-      "interface %s, version %u", registry, interface, version);
+      "name %" G_GUINT32_FORMAT ", interface %s, version %u", registry, name,
+      interface, version);
 
   if (g_strcmp0 (interface, "wl_compositor") == 0) {
     window_wayland->display.compositor =
@@ -518,8 +524,20 @@ registry_handle_global (void *data, struct wl_registry *registry,
   }
 }
 
+static void
+registry_handle_global_remove (void *data, struct wl_registry *registry,
+    uint32_t name)
+{
+  GstGLWindowWaylandEGL *window_wayland = data;
+
+  /* TODO: deal with any registry objects that may be removed */
+  GST_TRACE_OBJECT (window_wayland, "wl_registry %p global_remove %"
+      G_GUINT32_FORMAT, registry, name);
+}
+
 static const struct wl_registry_listener registry_listener = {
-  registry_handle_global
+  registry_handle_global,
+  registry_handle_global_remove,
 };
 
 static gboolean
@@ -548,8 +566,7 @@ gst_gl_window_wayland_egl_open (GstGLWindow * window, GError ** error)
    * each wayland resource we create as well as removing a race between
    * creation and the `wl_proxy_set_queue()` call. */
   window_egl->display.display = wl_proxy_create_wrapper (display->display);
-  window_egl->window.queue =
-      wl_display_create_queue (window_egl->display.display);
+  window_egl->window.queue = wl_display_create_queue (display->display);
 
   wl_proxy_set_queue ((struct wl_proxy *) window_egl->display.display,
       window_egl->window.queue);
@@ -623,10 +640,6 @@ _roundtrip_async (GstGLWindow * window)
 static void
 gst_gl_window_wayland_egl_show (GstGLWindow * window)
 {
-  GstGLWindowWaylandEGL *window_egl = GST_GL_WINDOW_WAYLAND_EGL (window);
-
-  create_surfaces (window_egl);
-
   gst_gl_window_send_message (window, (GstGLWindowCB) _roundtrip_async, window);
 }
 
