@@ -24,16 +24,12 @@
 #endif
 
 #include "gstglwindow_win32.h"
-#include "win32_message_source.h"
 
 LRESULT CALLBACK window_proc (HWND hWnd, UINT uMsg, WPARAM wParam,
     LPARAM lParam);
 LRESULT FAR PASCAL sub_class_proc (HWND hWnd, UINT uMsg, WPARAM wParam,
     LPARAM lParam);
 
-#define GST_GL_WINDOW_WIN32_GET_PRIVATE(o)  \
-  (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_TYPE_GL_WINDOW_WIN32, GstGLWindowWin32Private))
-
 enum
 {
   PROP_0
@@ -43,6 +39,7 @@ struct _GstGLWindowWin32Private
 {
   gint preferred_width;
   gint preferred_height;
+  GIOChannel *msg_io_channel;
 };
 
 #define GST_CAT_DEFAULT gst_gl_window_win32_debug
@@ -52,11 +49,12 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
   GST_DEBUG_CATEGORY_GET (GST_CAT_DEFAULT, "glwindow");
 #define gst_gl_window_win32_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstGLWindowWin32, gst_gl_window_win32,
-    GST_TYPE_GL_WINDOW, DEBUG_INIT);
+    GST_TYPE_GL_WINDOW, G_ADD_PRIVATE (GstGLWindowWin32) DEBUG_INIT);
 
 static void gst_gl_window_win32_set_window_handle (GstGLWindow * window,
     guintptr handle);
 static guintptr gst_gl_window_win32_get_display (GstGLWindow * window);
+static guintptr gst_gl_window_win32_get_window_handle (GstGLWindow * window);
 static void gst_gl_window_win32_set_preferred_size (GstGLWindow * window,
     gint width, gint height);
 static void gst_gl_window_win32_show (GstGLWindow * window);
@@ -64,30 +62,34 @@ static void gst_gl_window_win32_draw (GstGLWindow * window);
 gboolean gst_gl_window_win32_open (GstGLWindow * window, GError ** error);
 void gst_gl_window_win32_close (GstGLWindow * window);
 static void release_parent_win_id (GstGLWindowWin32 * window_win32);
+static void gst_gl_window_win32_send_message (GstGLWindow * window,
+    GstGLWindowCB callback, gpointer data);
 
 static void
 gst_gl_window_win32_class_init (GstGLWindowWin32Class * klass)
 {
   GstGLWindowClass *window_class = (GstGLWindowClass *) klass;
 
-  g_type_class_add_private (klass, sizeof (GstGLWindowWin32Private));
-
   window_class->set_window_handle =
       GST_DEBUG_FUNCPTR (gst_gl_window_win32_set_window_handle);
   window_class->draw = GST_DEBUG_FUNCPTR (gst_gl_window_win32_draw);
   window_class->get_display =
       GST_DEBUG_FUNCPTR (gst_gl_window_win32_get_display);
+  window_class->get_window_handle =
+      GST_DEBUG_FUNCPTR (gst_gl_window_win32_get_window_handle);
   window_class->set_preferred_size =
       GST_DEBUG_FUNCPTR (gst_gl_window_win32_set_preferred_size);
   window_class->show = GST_DEBUG_FUNCPTR (gst_gl_window_win32_show);
   window_class->open = GST_DEBUG_FUNCPTR (gst_gl_window_win32_open);
   window_class->close = GST_DEBUG_FUNCPTR (gst_gl_window_win32_close);
+  window_class->send_message =
+      GST_DEBUG_FUNCPTR (gst_gl_window_win32_send_message);
 }
 
 static void
 gst_gl_window_win32_init (GstGLWindowWin32 * window)
 {
-  window->priv = GST_GL_WINDOW_WIN32_GET_PRIVATE (window);
+  window->priv = gst_gl_window_win32_get_instance_private (window);
 }
 
 GstGLWindowWin32 *
@@ -95,8 +97,8 @@ gst_gl_window_win32_new (GstGLDisplay * display)
 {
   GstGLWindowWin32 *window;
 
-  if ((gst_gl_display_get_handle_type (display) & GST_GL_DISPLAY_TYPE_WIN32) ==
-      0)
+  if ((gst_gl_display_get_handle_type (display) &
+          (GST_GL_DISPLAY_TYPE_WIN32 | GST_GL_DISPLAY_TYPE_EGL)) == 0)
     /* we require an win32 display to create win32 windows */
     return NULL;
 
@@ -106,12 +108,19 @@ gst_gl_window_win32_new (GstGLDisplay * display)
   return window;
 }
 
-static void
-msg_cb (GstGLWindowWin32 * window_win32, MSG * msg, gpointer user_data)
+static gboolean
+msg_cb (GIOChannel * source, GIOCondition condition, gpointer data)
 {
+  MSG msg;
+
+  if (!PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
+    return G_SOURCE_CONTINUE;
+
   GST_TRACE ("handle message");
-  TranslateMessage (msg);
-  DispatchMessage (msg);
+  TranslateMessage (&msg);
+  DispatchMessage (&msg);
+
+  return G_SOURCE_CONTINUE;
 }
 
 gboolean
@@ -122,9 +131,11 @@ gst_gl_window_win32_open (GstGLWindow * window, GError ** error)
   if (!GST_GL_WINDOW_CLASS (parent_class)->open (window, error))
     return FALSE;
 
-  window_win32->msg_source = win32_message_source_new (window_win32);
-  g_source_set_callback (window_win32->msg_source, (GSourceFunc) msg_cb,
-      NULL, NULL);
+  window_win32->priv->msg_io_channel = g_io_channel_win32_new_messages (0);
+  window_win32->msg_source =
+      g_io_create_watch (window_win32->priv->msg_io_channel, G_IO_IN);
+  g_source_set_callback (window_win32->msg_source, (GSourceFunc) msg_cb, NULL,
+      NULL);
   g_source_attach (window_win32->msg_source, window->main_context);
 
   return TRUE;
@@ -150,6 +161,8 @@ gst_gl_window_win32_close (GstGLWindow * window)
   g_source_destroy (window_win32->msg_source);
   g_source_unref (window_win32->msg_source);
   window_win32->msg_source = NULL;
+  g_io_channel_unref (window_win32->priv->msg_io_channel);
+  window_win32->priv->msg_io_channel = NULL;
 
   GST_GL_WINDOW_CLASS (parent_class)->close (window);
 }
@@ -314,6 +327,16 @@ gst_gl_window_win32_get_display (GstGLWindow * window)
   return (guintptr) window_win32->device;
 }
 
+static guintptr
+gst_gl_window_win32_get_window_handle (GstGLWindow * window)
+{
+  GstGLWindowWin32 *window_win32;
+
+  window_win32 = GST_GL_WINDOW_WIN32 (window);
+
+  return (guintptr) window_win32->internal_win_id;
+}
+
 static void
 gst_gl_window_win32_set_window_handle (GstGLWindow * window, guintptr id)
 {
@@ -384,6 +407,131 @@ gst_gl_window_win32_draw (GstGLWindow * window)
       RDW_NOERASE | RDW_INTERNALPAINT | RDW_INVALIDATE);
 }
 
+typedef struct
+{
+  GstGLWindow *window;
+  const gchar *event_type;
+  gchar *key_string;
+} GstGLWindowWin32KeyEvent;
+
+static gboolean
+gst_gl_window_win32_handle_key_event_func (GstGLWindowWin32KeyEvent * event)
+{
+  gst_gl_window_send_key_event (event->window,
+      event->event_type, event->key_string);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+gst_gl_window_win32_key_event_free (GstGLWindowWin32KeyEvent * event)
+{
+  g_free (event->key_string);
+  g_free (event);
+}
+
+static void
+gst_gl_window_win32_handle_key_event (GstGLWindow * window, UINT uMsg,
+    LPARAM lParam)
+{
+  gunichar2 wcrep[128];
+
+  if (GetKeyNameTextW (lParam, (LPWSTR) wcrep, 128)) {
+    gchar *utfrep = g_utf16_to_utf8 (wcrep, 128, NULL, NULL, NULL);
+    if (utfrep) {
+      GstGLDisplay *display = window->display;
+      GstGLWindowWin32KeyEvent *key_event;
+
+      key_event = g_new0 (GstGLWindowWin32KeyEvent, 1);
+      key_event->window = window;
+      if (uMsg == WM_KEYDOWN)
+        key_event->event_type = "key-press";
+      else
+        key_event->event_type = "key-release";
+      key_event->key_string = utfrep;
+
+      g_main_context_invoke_full (display->main_context, G_PRIORITY_DEFAULT,
+          (GSourceFunc) gst_gl_window_win32_handle_key_event_func,
+          key_event, (GDestroyNotify) gst_gl_window_win32_key_event_free);
+    }
+  }
+}
+
+typedef struct
+{
+  GstGLWindow *window;
+  const gchar *event_type;
+  gint button;
+  gdouble pos_x;
+  gdouble pos_y;
+} GstGLWindowWin32MouseEvent;
+
+static gboolean
+gst_gl_window_win32_handle_mouse_event_func (GstGLWindowWin32MouseEvent * event)
+{
+  gst_gl_window_send_mouse_event (event->window,
+      event->event_type, event->button, event->pos_x, event->pos_y);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+gst_gl_window_win32_handle_mouse_event (GstGLWindow * window, UINT uMsg,
+    LPARAM lParam)
+{
+  gint button;
+  const gchar *event = NULL;
+
+  switch (uMsg) {
+    case WM_MOUSEMOVE:
+      button = 0;
+      event = "mouse-move";
+      break;
+    case WM_LBUTTONDOWN:
+      button = 1;
+      event = "mouse-button-press";
+      break;
+    case WM_LBUTTONUP:
+      button = 1;
+      event = "mouse-button-release";
+      break;
+    case WM_RBUTTONDOWN:
+      button = 2;
+      event = "mouse-button-press";
+      break;
+    case WM_RBUTTONUP:
+      button = 2;
+      event = "mouse-button-release";
+      break;
+    case WM_MBUTTONDOWN:
+      button = 3;
+      event = "mouse-button-press";
+      break;
+    case WM_MBUTTONUP:
+      button = 3;
+      event = "mouse-button-release";
+      break;
+    default:
+      break;
+  }
+
+  if (event) {
+    GstGLDisplay *display = window->display;
+    GstGLWindowWin32MouseEvent *mouse_event;
+
+    mouse_event = g_new0 (GstGLWindowWin32MouseEvent, 1);
+    mouse_event->window = window;
+    mouse_event->event_type = event;
+    mouse_event->button = button;
+    mouse_event->pos_x = (gdouble) LOWORD (lParam);
+    mouse_event->pos_y = (gdouble) HIWORD (lParam);
+
+    g_main_context_invoke_full (display->main_context, G_PRIORITY_DEFAULT,
+        (GSourceFunc) gst_gl_window_win32_handle_mouse_event_func,
+        mouse_event, (GDestroyNotify) g_free);
+  }
+}
+
 /* PRIVATE */
 
 LRESULT CALLBACK
@@ -464,12 +612,26 @@ window_proc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
         ret = TRUE;
         break;
       }
+      case WM_KEYDOWN:
+      case WM_KEYUP:
+        gst_gl_window_win32_handle_key_event (window, uMsg, lParam);
+        ret = DefWindowProc (hWnd, uMsg, wParam, lParam);
+        break;
+      case WM_LBUTTONDOWN:
+      case WM_LBUTTONUP:
+      case WM_RBUTTONDOWN:
+      case WM_RBUTTONUP:
+      case WM_MBUTTONDOWN:
+      case WM_MBUTTONUP:
+      case WM_MOUSEMOVE:
+        gst_gl_window_win32_handle_mouse_event (window, uMsg, lParam);
+        /* DefWindowProc will not chain up mouse event to parent window */
+        if (window_win32->parent_win_id)
+          PostMessage (window_win32->parent_win_id, uMsg, wParam, lParam);
+        ret = DefWindowProc (hWnd, uMsg, wParam, lParam);
+        break;
       default:
       {
-        /* transmit messages to the parrent (ex: mouse/keyboard input) */
-        HWND parent_id = window_win32->parent_win_id;
-        if (parent_id)
-          PostMessage (parent_id, uMsg, wParam, lParam);
         ret = DefWindowProc (hWnd, uMsg, wParam, lParam);
       }
     }
@@ -494,3 +656,37 @@ sub_class_proc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 
   return CallWindowProc (window_parent_proc, hWnd, uMsg, wParam, lParam);
 }
+
+typedef struct _GstGLWin32SyncMessage
+{
+  GstGLWindowCB callback;
+  gpointer data;
+
+  HANDLE *event;
+} GstGLWin32SyncMessage;
+
+static void
+_run_message_sync (GstGLWin32SyncMessage * message)
+{
+  if (message->callback)
+    message->callback (message->data);
+
+  SetEvent (message->event);
+}
+
+void
+gst_gl_window_win32_send_message (GstGLWindow * window,
+    GstGLWindowCB callback, gpointer data)
+{
+  GstGLWin32SyncMessage message;
+
+  message.callback = callback;
+  message.data = data;
+  message.event = CreateEvent (NULL, FALSE, FALSE, NULL);
+
+  gst_gl_window_send_message_async (window, (GstGLWindowCB) _run_message_sync,
+      &message, NULL);
+
+  WaitForSingleObject (message.event, INFINITE);
+  CloseHandle (message.event);
+}
