@@ -17,6 +17,64 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+/**
+ * SECTION:encoding-target
+ *
+ * On top of the notion of profiles, we implement the notion of EncodingTarget.
+ * Encoding Targets are basically a higher level of abstraction to define formats
+ * for specific target types. Those can define several GstEncodingProfiles with
+ * different names, for example one for transcoding in full HD, another one for
+ * low res, etc.. which are defined in the same encoding target.
+ *
+ * Basically if you want to encode a stream to send it to, say, youtube you should
+ * have a Youtube encoding target defined in the "online-service" category.
+ *
+ * ## Encoding target serialization format
+ *
+ * Encoding targets are serialized in a KeyFile like files.
+ *
+ * |[
+ * [GStreamer Encoding Target]
+ * name : <name>
+ * category : <category>
+ * \description : <description> #translatable
+ *
+ * [profile-<profile1name>]
+ * name : <name>
+ * \description : <description> #optional
+ * format : <format>
+ * preset : <preset>
+ *
+ * [streamprofile-<id>]
+ * parent : <encodingprofile.name>[,<encodingprofile.name>..]
+ * \type : <type> # "audio", "video", "text"
+ * format : <format>
+ * preset : <preset>
+ * restriction : <restriction>
+ * presence : <presence>
+ * pass : <pass>
+ * variableframerate : <variableframerate>
+ * ]|
+ *
+ * ## Location of encoding target files
+ *
+ * $GST_DATADIR/gstreamer-GST_API_VERSION/encoding-profile
+ * $HOME/gstreamer-GST_API_VERSION/encoding-profile
+ *
+ * There also is a GST_ENCODING_TARGET_PATH environment variable
+ * defining a list of folder containing encoding target files.
+ *
+ * ## Naming convention
+ *
+ * |[
+ *   $(target.category)/$(target.name).gep
+ * ]|
+ *
+ * ## Naming restrictions:
+ *
+ *  * lowercase ASCII letter for the first character
+ *  * Same for all other characters + numerics + hyphens
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,51 +84,30 @@
 #include <errno.h>
 #include <string.h>
 #include "encoding-target.h"
+#include "pbutils-private.h"
 
-/*
- * File format
- *
- * GKeyFile style.
- *
- * [GStreamer Encoding Target]
- * name : <name>
- * category : <category>
- * description : <description> #translatable
- *
- * [profile-<profile1name>]
- * name : <name>
- * description : <description> #optional
- * format : <format>
- * preset : <preset>
- *
- * [streamprofile-<id>]
- * parent : <encodingprofile.name>[,<encodingprofile.name>..]
- * type : <type> # "audio", "video", "text"
- * format : <format>
- * preset : <preset>
- * restriction : <restriction>
- * presence : <presence>
- * pass : <pass>
- * variableframerate : <variableframerate>
- *  */
+/* Documented in encoding-profile.c */
 
-/*
- * Location of profile files
- *
- * $GST_DATADIR/gstreamer-GST_API_VERSION/encoding-profile
- * $HOME/gstreamer-GST_API_VERSION/encoding-profile
- *
- * There also is a GST_ENCODING_TARGET_PATH environment variable
- * defining a list of folder containing encoding target files.
- *
- * Naming convention
- *   $(target.category)/$(target.name).gep
- *
- * Naming restrictions:
- *  lowercase ASCII letter for the first character
- *  Same for all other characters + numerics + hyphens
- */
+#ifndef GST_DISABLE_GST_DEBUG
+#define GST_CAT_DEFAULT gst_pb_utils_encoding_target_ensure_debug_category()
 
+static GstDebugCategory *
+gst_pb_utils_encoding_target_ensure_debug_category (void)
+{
+  static gsize cat_gonce = 0;
+
+  if (g_once_init_enter (&cat_gonce)) {
+    GstDebugCategory *cat = NULL;
+
+    GST_DEBUG_CATEGORY_INIT (cat, "encoding-target", 0,
+        "GstPbUtils encoding target");
+
+    g_once_init_leave (&cat_gonce, (gsize) cat);
+  }
+
+  return (GstDebugCategory *) cat_gonce;
+}
+#endif /* GST_DISABLE_GST_DEBUG */
 
 #define GST_ENCODING_TARGET_HEADER "GStreamer Encoding Target"
 #define GST_ENCODING_TARGET_DIRECTORY "encoding-profiles"
@@ -83,6 +120,7 @@ struct _GstEncodingTarget
   gchar *name;
   gchar *category;
   gchar *description;
+  gchar *path;
   GList *profiles;
 
   /*< private > */
@@ -107,6 +145,7 @@ gst_encoding_target_finalize (GObject * object)
   g_free (target->name);
   g_free (target->category);
   g_free (target->description);
+  g_free (target->path);
 
   g_list_foreach (target->profiles, (GFunc) g_object_unref, NULL);
   g_list_free (target->profiles);
@@ -155,6 +194,21 @@ gst_encoding_target_get_description (GstEncodingTarget * target)
   return target->description;
 }
 
+/**
+ * gst_encoding_target_get_path:
+ * @target: a #GstEncodingTarget
+ *
+ * Returns: (transfer none) (nullable) (type filename): The path to the @target file.
+ *
+ * Since: 1.18
+ */
+const gchar *
+gst_encoding_target_get_path (GstEncodingTarget * target)
+{
+  g_return_val_if_fail (GST_IS_ENCODING_TARGET (target), NULL);
+  return target->path;
+}
+
 /**
  * gst_encoding_target_get_profiles:
  * @target: a #GstEncodingTarget
@@ -173,7 +227,7 @@ gst_encoding_target_get_profiles (GstEncodingTarget * target)
  * @target: a #GstEncodingTarget
  * @name: the name of the profile to retrieve
  *
- * Returns: (transfer full): The matching #GstEncodingProfile, or %NULL.
+ * Returns: (nullable) (transfer full): The matching #GstEncodingProfile, or %NULL.
  */
 GstEncodingProfile *
 gst_encoding_target_get_profile (GstEncodingTarget * target, const gchar * name)
@@ -222,6 +276,10 @@ validate_name (const gchar * name)
     /* if an hyphen, continue */
     if (name[i] == '-')
       continue;
+    /* if an ';', continue (list delimiter) */
+    if (name[i] == ';') {
+      continue;
+    }
     /* remaining should only be ascii letters */
     if (!g_ascii_isalpha (name[i]))
       return FALSE;
@@ -246,12 +304,12 @@ validate_name (const gchar * name)
  * first character, followed by either lowercase ASCII letters, digits or
  * hyphens ('-').
  *
- * The @category <emphasis>should</emphasis> be one of the existing
+ * The @category *should* be one of the existing
  * well-defined categories, like #GST_ENCODING_CATEGORY_DEVICE, but it
- * <emphasis>can</emphasis> be a application or user specific category if
+ * *can* be a application or user specific category if
  * needed.
  *
- * Returns: (transfer full): The newly created #GstEncodingTarget or %NULL if
+ * Returns: (transfer full) (nullable): The newly created #GstEncodingTarget or %NULL if
  * there was an error.
  */
 
@@ -407,6 +465,8 @@ get_locale (void)
   const char *loc = NULL;
   gchar *ret;
 
+  gst_pb_utils_init_locale_text_domain ();
+
 #ifdef ENABLE_NLS
 #if defined(LC_MESSAGES)
   loc = setlocale (LC_MESSAGES, NULL);
@@ -526,7 +586,8 @@ parse_encoding_profile (GKeyFile * in, gchar * parentprofilename,
 {
   GstEncodingProfile *sprof = NULL;
   gchar **parent;
-  gchar *proftype, *format, *preset, *restriction, *pname, *description;
+  gchar *proftype, *format, *preset, *restriction, *pname, *description,
+      *locale;
   GstCaps *formatcaps = NULL;
   GstCaps *restrictioncaps = NULL;
   gboolean variableframerate;
@@ -564,21 +625,11 @@ parse_encoding_profile (GKeyFile * in, gchar * parentprofilename,
 
   pname = g_key_file_get_value (in, profilename, "name", NULL);
 
-  /* First try to get localized description */
-  {
-    gchar *locale;
-
-    locale = get_locale ();
-    if (locale != NULL) {
-      /* will try to fall back to untranslated string if no translation found */
-      description = g_key_file_get_locale_string (in, profilename,
-          "description", locale, NULL);
-      g_free (locale);
-    } else {
-      description =
-          g_key_file_get_string (in, profilename, "description", NULL);
-    }
-  }
+  locale = get_locale ();
+  /* will try to fall back to untranslated string if no translation found */
+  description = g_key_file_get_locale_string (in, profilename,
+      "description", locale, NULL);
+  g_free (locale);
 
   /* Note: a missing description is normal for non-container profiles */
   if (description == NULL) {
@@ -744,8 +795,8 @@ empty_name:
 
 /**
  * gst_encoding_target_load_from_file:
- * @filepath: The file location to load the #GstEncodingTarget from
- * @error: If an error occured, this field will be filled in.
+ * @filepath: (type filename): The file location to load the #GstEncodingTarget from
+ * @error: If an error occurred, this field will be filled in.
  *
  * Opens the provided file and returns the contained #GstEncodingTarget.
  *
@@ -844,10 +895,12 @@ gst_encoding_target_subload (gchar * path, const gchar * category,
 
 /**
  * gst_encoding_target_load:
- * @name: the name of the #GstEncodingTarget to load.
- * @category: (allow-none): the name of the target category, like
+ * @name: the name of the #GstEncodingTarget to load (automatically
+ * converted to lower case internally as capital letters are not
+ * valid for target names).
+ * @category: (nullable): the name of the target category, like
  * #GST_ENCODING_CATEGORY_DEVICE. Can be %NULL
- * @error: If an error occured, this field will be filled in.
+ * @error: If an error occurred, this field will be filled in.
  *
  * Searches for the #GstEncodingTarget with the given name, loads it
  * and returns it.
@@ -862,19 +915,23 @@ gst_encoding_target_load (const gchar * name, const gchar * category,
     GError ** error)
 {
   gint i;
-  gchar *lfilename, *tldir, **encoding_target_dirs;
+  gchar *p, *lname, *lfilename = NULL, *tldir, **encoding_target_dirs;
   const gchar *envvar;
   GstEncodingTarget *target = NULL;
 
   g_return_val_if_fail (name != NULL, NULL);
 
-  if (!validate_name (name))
+  p = lname = g_str_to_ascii (name, NULL);
+  for (; *p; ++p)
+    *p = g_ascii_tolower (*p);
+
+  if (!validate_name (lname))
     goto invalid_name;
 
   if (category && !validate_name (category))
     goto invalid_category;
 
-  lfilename = g_strdup_printf ("%s" GST_ENCODING_TARGET_SUFFIX, name);
+  lfilename = g_strdup_printf ("%s" GST_ENCODING_TARGET_SUFFIX, lname);
 
   envvar = g_getenv ("GST_ENCODING_TARGET_PATH");
   if (envvar) {
@@ -908,28 +965,55 @@ gst_encoding_target_load (const gchar * name, const gchar * category,
     g_free (tldir);
   }
 
+  if (!target) {
+    GList *tmp, *targets = gst_encoding_list_all_targets (NULL);
+
+    for (tmp = targets; tmp; tmp = tmp->next) {
+      gint i;
+      GstEncodingTarget *tmptarget = tmp->data;
+      gchar **names = g_strsplit (tmptarget->name, ";", -1);
+
+      for (i = 0; names[i]; i++) {
+        if (!g_strcmp0 (names[i], lname) && (!category ||
+                !g_strcmp0 (tmptarget->category, category))) {
+          target = gst_object_ref (tmptarget);
+
+          break;
+        }
+      }
+      g_strfreev (names);
+
+      if (target)
+        break;
+    }
+
+    g_list_free_full (targets, gst_object_unref);
+  }
+
+
 done:
   g_free (lfilename);
+  g_free (lname);
 
   return target;
 
 invalid_name:
   {
-    GST_ERROR ("Invalid name for encoding target : '%s'", name);
-    return NULL;
+    GST_INFO ("Invalid name for encoding target : '%s'", name);
+    goto done;
   }
 invalid_category:
   {
-    GST_ERROR ("Invalid name for encoding category : '%s'", category);
-    return NULL;
+    GST_INFO ("Invalid name for encoding category : '%s'", category);
+    goto done;
   }
 }
 
 /**
  * gst_encoding_target_save_to_file:
  * @target: a #GstEncodingTarget
- * @filepath: the location to store the @target at.
- * @error: If an error occured, this field will be filled in.
+ * @filepath: (type filename): the location to store the @target at.
+ * @error: If an error occurred, this field will be filled in.
  *
  * Saves the @target to the provided file location.
  *
@@ -964,6 +1048,8 @@ gst_encoding_target_save_to_file (GstEncodingTarget * target,
   if (!g_file_set_contents (filepath, data, data_size, error))
     goto write_failed;
 
+  target->path = g_strdup (filepath);
+
   g_key_file_free (out);
   g_free (data);
 
@@ -996,7 +1082,7 @@ write_failed:
 /**
  * gst_encoding_target_save:
  * @target: a #GstEncodingTarget
- * @error: If an error occured, this field will be filled in.
+ * @error: If an error occurred, this field will be filled in.
  *
  * Saves the @target to a default user-local directory.
  *
@@ -1094,7 +1180,7 @@ gst_encoding_list_available_categories (void)
     else
       g_free (name);
   }
-  g_free (tmp1);
+  g_list_free (tmp1);
 
   return res;
 }
@@ -1121,10 +1207,10 @@ sub_get_all_targets (gchar * subdir)
     fullname = g_build_filename (subdir, filename, NULL);
     target = gst_encoding_target_load_from_file (fullname, NULL);
     if (target) {
+      target->path = fullname;
       res = g_list_append (res, target);
     } else
       GST_WARNING ("Failed to get a target from %s", fullname);
-    g_free (fullname);
   }
   g_dir_close (dir);
 
@@ -1165,16 +1251,36 @@ get_all_targets (gchar * topdir, const gchar * categoryname)
 static guint
 compare_targets (const GstEncodingTarget * ta, const GstEncodingTarget * tb)
 {
-  if (!g_strcmp0 (ta->name, tb->name)
-      && !g_strcmp0 (ta->category, tb->category))
+  if (g_strcmp0 (ta->name, tb->name)
+      || g_strcmp0 (ta->category, tb->category))
     return -1;
 
   return 0;
 }
 
+static GList *
+merge_targets (GList * res, GList * extra)
+{
+  GList *tmp;
+
+  /* FIXME : We should merge the system-wide profiles into the user-locals
+   * instead of stopping at identical target names */
+  for (tmp = extra; tmp; tmp = tmp->next) {
+    GstEncodingTarget *target = (GstEncodingTarget *) tmp->data;
+    if (g_list_find_custom (res, target, (GCompareFunc) compare_targets))
+      gst_encoding_target_unref (target);
+    else
+      res = g_list_append (res, target);
+  }
+
+  g_list_free (extra);
+
+  return res;
+}
+
 /**
  * gst_encoding_list_all_targets:
- * @categoryname: (allow-none): The category, for ex: #GST_ENCODING_CATEGORY_DEVICE.
+ * @categoryname: (nullable): The category, for ex: #GST_ENCODING_CATEGORY_DEVICE.
  * Can be %NULL.
  *
  * List all available #GstEncodingTarget for the specified category, or all categories
@@ -1185,34 +1291,35 @@ compare_targets (const GstEncodingTarget * ta, const GstEncodingTarget * tb)
 GList *
 gst_encoding_list_all_targets (const gchar * categoryname)
 {
-  GList *res;
-  GList *tmp1, *tmp2;
+  GList *res = NULL;
   gchar *topdir;
+  gchar **encoding_target_dirs;
+
+  const gchar *envvar = g_getenv ("GST_ENCODING_TARGET_PATH");
+  if (envvar) {
+    gint i;
+
+    encoding_target_dirs = g_strsplit (envvar, G_SEARCHPATH_SEPARATOR_S, -1);
+    for (i = 0; encoding_target_dirs[i]; i++)
+      res =
+          merge_targets (res, get_all_targets (encoding_target_dirs[i],
+              categoryname));
+
+    g_strfreev (encoding_target_dirs);
+  }
 
   /* Get user-locals */
   topdir =
       g_build_filename (g_get_user_data_dir (), "gstreamer-" GST_API_VERSION,
       GST_ENCODING_TARGET_DIRECTORY, NULL);
-  res = get_all_targets (topdir, categoryname);
+  res = merge_targets (res, get_all_targets (topdir, categoryname));
   g_free (topdir);
 
   /* Get system-wide */
   topdir = g_build_filename (GST_DATADIR, "gstreamer-" GST_API_VERSION,
       GST_ENCODING_TARGET_DIRECTORY, NULL);
-  tmp1 = get_all_targets (topdir, categoryname);
+  res = merge_targets (res, get_all_targets (topdir, categoryname));
   g_free (topdir);
 
-  /* Merge system-wide targets */
-  /* FIXME : We should merge the system-wide profiles into the user-locals
-   * instead of stopping at identical target names */
-  for (tmp2 = tmp1; tmp2; tmp2 = tmp2->next) {
-    GstEncodingTarget *target = (GstEncodingTarget *) tmp2->data;
-    if (g_list_find_custom (res, target, (GCompareFunc) compare_targets))
-      gst_encoding_target_unref (target);
-    else
-      res = g_list_append (res, target);
-  }
-  g_list_free (tmp1);
-
   return res;
 }
