@@ -64,7 +64,7 @@
 
 #include <gst/base/gsttypefindhelper.h>
 #include <gst/base/gstadapter.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
 typedef enum
@@ -102,6 +102,7 @@ struct _GstTagDemuxPrivate
 
   GstSegment segment;
   gboolean need_newseg;
+  guint32 segment_seqnum;
 
   guint64 offset;
 
@@ -265,6 +266,7 @@ gst_tag_demux_reset (GstTagDemux * tagdemux)
 
   gst_segment_init (&tagdemux->priv->segment, GST_FORMAT_UNDEFINED);
   tagdemux->priv->need_newseg = TRUE;
+  tagdemux->priv->segment_seqnum = gst_util_seqnum_next ();
 
   g_list_foreach (tagdemux->priv->pending_events,
       (GFunc) gst_mini_object_unref, NULL);
@@ -377,7 +379,8 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref,
   guint trim_start = 0;
   guint out_size, bsize;
   guint64 out_offset, boffset;
-  gboolean need_sub = FALSE;
+  gboolean need_fixup = FALSE;
+  gboolean is_writable;
 
   bsize = out_size = gst_buffer_get_size (buf);
   boffset = out_offset = GST_BUFFER_OFFSET (buf);
@@ -402,7 +405,7 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref,
 
       if (out_offset + out_size > v1tag_offset) {
         out_size = v1tag_offset - out_offset;
-        need_sub = TRUE;
+        need_fixup = TRUE;
       }
     }
   }
@@ -421,11 +424,16 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref,
     } else {
       out_offset -= tagdemux->priv->strip_start;
     }
-    need_sub = TRUE;
+    need_fixup = TRUE;
   }
 
-  if (need_sub) {
-    if (out_size != bsize || !gst_buffer_is_writable (buf)) {
+  if (!need_fixup)
+    goto done;
+
+  is_writable = gst_buffer_is_writable (buf);
+
+  if (out_size != bsize || !is_writable) {
+    if (!is_writable) {
       GstBuffer *sub;
 
       GST_DEBUG_OBJECT (tagdemux, "Sub-buffering to trim size %d offset %"
@@ -444,15 +452,23 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref,
       *buf_ref = buf = sub;
       *buf_size = out_size;
     } else {
-      GST_DEBUG_OBJECT (tagdemux, "Adjusting buffer from size %d offset %"
+      GST_DEBUG_OBJECT (tagdemux, "Resizing buffer to trim size %d offset %"
           G_GINT64_FORMAT " to %d offset %" G_GINT64_FORMAT,
           bsize, boffset, out_size, out_offset);
-    }
 
-    GST_BUFFER_OFFSET (buf) = out_offset;
-    GST_BUFFER_OFFSET_END (buf) = out_offset + out_size;
+      gst_buffer_resize (buf, trim_start, out_size);
+    }
+  } else {
+    GST_DEBUG_OBJECT (tagdemux, "Adjusting buffer from size %d offset %"
+        G_GINT64_FORMAT " to %d offset %" G_GINT64_FORMAT,
+        bsize, boffset, out_size, out_offset);
   }
 
+  GST_BUFFER_OFFSET (buf) = out_offset;
+  GST_BUFFER_OFFSET_END (buf) = out_offset + out_size;
+
+done:
+
   return TRUE;
 
 no_out_buffer_end:
@@ -641,9 +657,11 @@ gst_tag_demux_chain_buffer (GstTagDemux * demux, GstBuffer * buf,
 
       /* Trim the buffer and adjust offset for typefinding */
       typefind_buf = demux->priv->collect;
-      gst_buffer_ref (typefind_buf);
-      if (!gst_tag_demux_trim_buffer (demux, &typefind_buf, &typefind_size))
-        return GST_FLOW_EOS;
+      if (typefind_buf) {
+        gst_buffer_ref (typefind_buf);
+        if (!gst_tag_demux_trim_buffer (demux, &typefind_buf, &typefind_size))
+          return GST_FLOW_EOS;
+      }
 
       if (typefind_buf == NULL)
         break;                  /* Still need more data */
@@ -753,6 +771,7 @@ gst_tag_demux_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     case GST_EVENT_SEGMENT:
     {
       gst_event_copy_segment (event, &demux->priv->segment);
+      demux->priv->segment_seqnum = gst_event_get_seqnum (event);
 
       demux->priv->need_newseg = TRUE;
       gst_event_unref (event);
@@ -1838,6 +1857,7 @@ gst_tag_demux_send_new_segment (GstTagDemux * tagdemux)
   /* Can't adjust segments in non-BYTES formats */
   if (tagdemux->priv->segment.format != GST_FORMAT_BYTES) {
     event = gst_event_new_segment (seg);
+    gst_event_set_seqnum (event, tagdemux->priv->segment_seqnum);
     return gst_pad_push_event (tagdemux->priv->srcpad, event);
   }
 
@@ -1894,6 +1914,7 @@ gst_tag_demux_send_new_segment (GstTagDemux * tagdemux)
   newseg.stop = stop;
   newseg.time = time;
   event = gst_event_new_segment (&newseg);
+  gst_event_set_seqnum (event, tagdemux->priv->segment_seqnum);
 
   return gst_pad_push_event (tagdemux->priv->srcpad, event);
 }
