@@ -109,7 +109,7 @@ id3v2_parse_frame (ID3TagsWorking * work)
 
   if (work->frame_flags & (ID3V2_FRAME_FORMAT_COMPRESSION |
           ID3V2_FRAME_FORMAT_DATA_LENGTH_INDICATOR)) {
-    if (work->hdr.frame_data_size <= 4)
+    if (frame_data_size <= 4)
       return FALSE;
     if (ID3V2_VER_MAJOR (work->hdr.version) == 3) {
       work->parse_size = GST_READ_UINT32_BE (frame_data);
@@ -1043,7 +1043,7 @@ string_utf8_dup (const gchar * start, const guint size)
       if ((utf8 =
               g_convert (start, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
         if (bytes_read == size) {
-          GST_DEBUG ("Using charset %s to interperate id3 tags\n", *c);
+          GST_DEBUG ("Using charset %s to interpret id3 tags", *c);
           g_strfreev (csets);
           goto beach;
         }
@@ -1103,11 +1103,16 @@ parse_insert_string_field (guint8 encoding, gchar * data, gint data_size,
       /* Sometimes we see strings with multiple BOM markers at the start.
        * In that case, we assume the innermost one is correct. If that fails
        * to produce valid UTF-8, we try the other endianness anyway */
-      while (data_size > 2 && find_utf16_bom (data, &data_endianness)) {
+      while (data_size >= 2 && find_utf16_bom (data, &data_endianness)) {
         data += 2;              /* skip BOM */
         data_size -= 2;
       }
 
+      if (data_size < 2) {
+        field = g_strdup ("");
+        break;
+      }
+
       /* alloc needed to ensure correct alignment which is required by GLib */
       len = data_size / 2;
       utf16 = g_try_new (gunichar2, len + 1);
@@ -1193,7 +1198,7 @@ parse_split_strings (guint8 encoding, gchar * data, gint data_size,
       for (text_pos = 0; text_pos < data_size; text_pos++) {
         if (data[text_pos] == 0) {
           parse_insert_string_field (encoding, data + prev,
-              text_pos - prev + 1, fields);
+              text_pos - prev, fields);
           prev = text_pos + 1;
         }
       }
@@ -1207,7 +1212,7 @@ parse_split_strings (guint8 encoding, gchar * data, gint data_size,
       for (prev = 0, text_pos = 0; text_pos < data_size; text_pos++) {
         if (data[text_pos] == '\0') {
           parse_insert_string_field (encoding, data + prev,
-              text_pos - prev + 1, fields);
+              text_pos - prev, fields);
           prev = text_pos + 1;
         }
       }
@@ -1224,10 +1229,8 @@ parse_split_strings (guint8 encoding, gchar * data, gint data_size,
         if (data[text_pos] == '\0' && data[text_pos + 1] == '\0') {
           /* found a delimiter */
           parse_insert_string_field (encoding, data + prev,
-              text_pos - prev + 2, fields);
-          text_pos++;           /* Advance to the 2nd NULL terminator */
-          prev = text_pos + 1;
-          break;
+              text_pos - prev, fields);
+          prev = text_pos + 2;
         }
       }
       if (data_size - prev > 1 &&
