@@ -29,7 +29,7 @@
 #include "gstsmartencoder.h"
 #include "gststreamsplitter.h"
 #include "gststreamcombiner.h"
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 
 /**
  * SECTION:element-encodebin
@@ -2144,7 +2144,7 @@ _get_muxer (GstEncodeBaseBin * ebin)
       GstCaps *sformat = gst_encoding_profile_get_format (sprof);
 
       if (!_factory_can_handle_caps (muxerfact, sformat, GST_PAD_SINK, FALSE)) {
-        GST_ERROR ("Skipping muxer because it can't sink caps %"
+        GST_DEBUG ("Skipping muxer because it can't sink caps %"
             GST_PTR_FORMAT, sformat);
         cansinkstreams = FALSE;
         if (sformat)
@@ -2220,6 +2220,7 @@ create_elements_and_pads (GstEncodeBaseBin * ebin)
       if (!pad)
         goto no_muxer_ghost_pad;
 
+      gst_object_unref (muxerpad);
       gst_element_add_pad (GST_ELEMENT (ebin), pad);
     }
 
