@@ -24,12 +24,17 @@
 #include "config.h"
 #endif
 
+#include <ctype.h>              /* isspace() */
+#include <stdio.h>              /* sscanf() */
 #include <stdlib.h>             /* atoi() */
 #include <string.h>
 
 #include "gstssaparse.h"
+#include "gstsubparseelements.h"
+
 
 GST_DEBUG_CATEGORY_STATIC (ssa_parse_debug);
+#undef GST_CAT_DEFAULT
 #define GST_CAT_DEFAULT ssa_parse_debug
 
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -46,6 +51,9 @@ static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_ssa_parse_parent_class parent_class
 G_DEFINE_TYPE (GstSsaParse, gst_ssa_parse, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (ssaparse, "ssaparse",
+    GST_RANK_PRIMARY, GST_TYPE_SSA_PARSE, sub_parse_element_init (plugin));
+
 
 static GstStateChangeReturn gst_ssa_parse_change_state (GstElement *
     element, GstStateChange transition);
@@ -97,10 +105,8 @@ gst_ssa_parse_class_init (GstSsaParseClass * klass)
 
   object_class->dispose = gst_ssa_parse_dispose;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_templ));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_templ));
+  gst_element_class_add_static_pad_template (element_class, &sink_templ);
+  gst_element_class_add_static_pad_template (element_class, &src_templ);
   gst_element_class_set_static_metadata (element_class,
       "SSA Subtitle Parser", "Codec/Parser/Subtitle",
       "Parses SSA subtitle streams",
@@ -171,8 +177,10 @@ gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps)
 
   gst_buffer_ref (priv);
 
-  if (!gst_buffer_map (priv, &map, GST_MAP_READ))
+  if (!gst_buffer_map (priv, &map, GST_MAP_READ)) {
+    gst_buffer_unref (priv);
     return FALSE;
+  }
 
   GST_MEMDUMP_OBJECT (parse, "init section", map.data, map.size);
 
@@ -197,6 +205,8 @@ gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps)
   }
 
   /* FIXME: parse initial section */
+  if (parse->ini)
+    g_free (parse->ini);
   parse->ini = g_strndup (ptr, left);
   GST_LOG_OBJECT (parse, "Init section:\n%s", parse->ini);
 
