@@ -58,6 +58,7 @@ struct _GstVideoRate
   gboolean force_variable_rate;
   gboolean updating_caps;
   guint64 max_duplication_time;
+  guint64 max_closing_segment_duplication_duration;
 
   /* segment handling */
   GstSegment segment;
@@ -70,10 +71,19 @@ struct _GstVideoRate
   gboolean drop_only;
   guint64 average_period_set;
 
-  volatile int max_rate;
+  int max_rate;
   gdouble rate;
   gdouble pending_rate;
+
+  GstCaps *in_caps;
+  /* Only set right after caps were set so that we still have a reference to
+   * the caps matching the content of `->prevbuf`, this way, if we get an EOS
+   * right after a CAPS, we can reset to those caps and close the segment with
+   * it */
+  GstCaps *prev_caps;
 };
 
+GST_ELEMENT_REGISTER_DECLARE (videorate);
+
 G_END_DECLS
 #endif /* __GST_VIDEO_RATE_H__ */
