@@ -35,13 +35,11 @@
 #define TO_16(x) (((x)<<8) | (x))
 
 static unsigned char
-random_char (void)
+random_char (guint * state)
 {
-  static unsigned int state;
-
-  state *= 1103515245;
-  state += 12345;
-  return (state >> 16) & 0xff;
+  *state *= 1103515245;
+  *state += 12345;
+  return (*state >> 16) & 0xff;
 }
 
 enum
@@ -63,7 +61,7 @@ enum
 static const struct vts_color_struct vts_colors_bt709_ycbcr_100[] = {
   {235, 128, 128, 255, 255, 255, 255, (235 << 8)},
   {219, 16, 138, 255, 255, 255, 0, (219 << 8)},
-  {188, 154, 16, 255, 0, 255, 255, (188 < 8)},
+  {188, 154, 16, 255, 0, 255, 255, (188 << 8)},
   {173, 42, 26, 255, 0, 255, 0, (173 << 8)},
   {78, 214, 230, 255, 255, 0, 255, (78 << 8)},
   {63, 102, 240, 255, 255, 0, 0, (64 << 8)},
@@ -90,10 +88,45 @@ static const struct vts_color_struct vts_colors_bt709_ycbcr_75[] = {
   {32, 128, 128, 255, 19, 19, 19, (32 << 8)},
 };
 
+/* 8-bit values, the selection and order of the colors is done by the
+   drawing function */
+static const struct vts_color_struct vts_colors_bt709_ycbcr_rp_219[] = {
+  /* Pattern 1, smpte75 with 16:9 gray extensions */
+  {104, 128, 128, 255, 102, 102, 102, (104 << 8)},      /* row 0, 40% gray */
+  {180, 128, 128, 255, 191, 191, 191, (180 << 8)},      /* row 1, 75% white */
+  {168, 44, 136, 255, 190, 203, 8, (168 << 8)}, /* row 2, 75% yellow */
+  {145, 147, 44, 255, 16, 211, 189, (145 << 8)},        /* row 3, 75% cyan */
+  {133, 63, 52, 255, 15, 223, 5, (133 << 8)},   /* row 4, 75% green */
+  {63, 193, 204, 255, 176, 0, 186, (63 << 8)},  /* row 5, 75% magenta */
+  {51, 109, 212, 255, 175, 0, 2, (51 << 8)},    /* row 6, 75% red */
+  {28, 212, 120, 255, 1, 0, 183, (28 << 8)},    /* row 7, 75% blue */
+
+  /* Pattern 2 */
+  {188, 154, 16, 255, 22, 255, 253, (188 << 8)},        /* row 8, 100% cyan */
+  {235, 128, 128, 255, 255, 255, 255, (235 << 8)},      /* row 9, 100% white */
+  {61, 103, 157, 255, 99, 48, 15, (61 << 8)},   /* row 10, *2=+I */
+  {61, 153, 99, 255, 6, 66, 103, (61 << 8)},    /* row 11, *2=-I */
+  {32, 240, 118, 255, 3, 0, 245, (32 << 8)},    /* row 12, 100% blue */
+
+  /* Pattern 3 */
+  {219, 16, 138, 255, 252, 255, 10, (219 << 8)},        /* row 13, 100% yellow */
+  {16, 128, 128, 255, 0, 0, 0, (16 << 8)},      /* row 14, 0% black */
+  {35, 174, 152, 255, 60, 0, 115, (35 << 8)},   /* row 15, +Q */
+  {63, 102, 240, 255, 233, 0, 2, (63 << 8)},    /* row 16, 100% red */
+
+  /* Pattern 4 */
+  {49, 128, 128, 255, 38, 38, 38, (47 << 8)},   /* row 17, 15% gray */
+  {1, 128, 128, 255, 0, 0, 0, (1 << 8)},        /* row 18, sub-black valley */
+  {254, 128, 128, 255, 255, 255, 255, (254 << 8)},      /* row 19, super-white peak */
+  {12, 128, 128, 255, 0, 0, 0, (12 << 8)},      /* row 20, -2% black */
+  {20, 128, 128, 255, 5, 5, 5, (20 << 8)},      /* row 21, +2% black */
+  {25, 128, 128, 255, 10, 11, 10, (25 << 8)},   /* row 22, +4% black */
+};
+
 static const struct vts_color_struct vts_colors_bt601_ycbcr_100[] = {
   {235, 128, 128, 255, 255, 255, 255, (235 << 8)},
   {210, 16, 146, 255, 255, 255, 0, (219 << 8)},
-  {170, 166, 16, 255, 0, 255, 255, (188 < 8)},
+  {170, 166, 16, 255, 0, 255, 255, (188 << 8)},
   {145, 54, 34, 255, 0, 255, 0, (173 << 8)},
   {106, 202, 222, 255, 255, 0, 255, (78 << 8)},
   {81, 90, 240, 255, 255, 0, 0, (64 << 8)},
@@ -258,11 +291,21 @@ videotestsrc_convert_tmpline (paintinfo * p, GstVideoFrame * frame, int j)
   int height = frame->info.height;
   int n_lines = p->n_lines;
   int offset = p->offset;
+  guint8 *tmpline = p->tmpline, *tmpline2 = p->tmpline2;
 
+  /* `tmpline2` is only used here and when the shift amount `x` is
+   * non-zero, which only applies when `horizontal-speed` is set.
+   *
+   * Instead of copying the rotated line back to `tmpline` for
+   * `p->convert_tmpline` to use, swap the pointers briefly. Besides
+   * being cheaper, this also lets us reuse the painted `tmpline` for
+   * subsequent lines.
+   */
   if (x != 0) {
-    memcpy (p->tmpline2, p->tmpline, width * 4);
-    memcpy (p->tmpline, p->tmpline2 + x * 4, (width - x) * 4);
-    memcpy (p->tmpline + (width - x) * 4, p->tmpline2, x * 4);
+    memcpy (tmpline2, tmpline + x * 4, (width - x) * 4);
+    memcpy (tmpline2 + (width - x) * 4, tmpline, x * 4);
+    p->tmpline = tmpline2;
+    p->tmpline2 = tmpline;
   }
 
   for (i = width; i < width + 5; i++) {
@@ -280,6 +323,11 @@ videotestsrc_convert_tmpline (paintinfo * p, GstVideoFrame * frame, int j)
       p->convert_tmpline (p, frame, j);
     }
   }
+
+  if (x != 0) {
+    p->tmpline = tmpline;
+    p->tmpline2 = tmpline2;
+  }
 }
 
 #define BLEND1(a,b,x) ((a)*(x) + (b)*(255-(x)))
@@ -303,19 +351,20 @@ videotestsrc_blend_color (struct vts_color_struct *dest,
 #endif
 
 static void
-videotestsrc_blend_line (GstVideoTestSrc * v, guint8 * dest, guint8 * src,
-    struct vts_color_struct *a, struct vts_color_struct *b, int n)
+videotestsrc_blend_line (GstVideoTestSrc * v, guint8 * dest,
+    const guint8 * src, const struct vts_color_struct *a,
+    const struct vts_color_struct *b, int x1, int x2)
 {
   int i;
   if (v->bayer || GST_VIDEO_INFO_IS_RGB (&v->info)) {
-    for (i = 0; i < n; i++) {
+    for (i = x1; i < x2; i++) {
       dest[i * 4 + 0] = BLEND (a->A, b->A, src[i]);
       dest[i * 4 + 1] = BLEND (a->R, b->R, src[i]);
       dest[i * 4 + 2] = BLEND (a->G, b->G, src[i]);
       dest[i * 4 + 3] = BLEND (a->B, b->B, src[i]);
     }
   } else {
-    for (i = 0; i < n; i++) {
+    for (i = x1; i < x2; i++) {
       dest[i * 4 + 0] = BLEND (a->A, b->A, src[i]);
       dest[i * 4 + 1] = BLEND (a->Y, b->Y, src[i]);
       dest[i * 4 + 2] = BLEND (a->U, b->U, src[i]);
@@ -326,7 +375,8 @@ videotestsrc_blend_line (GstVideoTestSrc * v, guint8 * dest, guint8 * src,
 }
 
 void
-gst_video_test_src_smpte (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_smpte (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int y1, y2;
@@ -413,11 +463,11 @@ gst_video_test_src_smpte (GstVideoTestSrc * v, GstVideoFrame * frame)
       p->color = &color;
 
       for (i = x1; i < w; i++) {
-        int y = random_char ();
+        int y = random_char (&v->random_state);
         p->tmpline_u8[i] = y;
       }
-      videotestsrc_blend_line (v, p->tmpline + x1 * 4, p->tmpline_u8 + x1,
-          &p->foreground_color, &p->background_color, w - x1);
+      videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
+          &p->foreground_color, &p->background_color, x1, w);
 
     }
     videotestsrc_convert_tmpline (p, frame, j);
@@ -426,7 +476,342 @@ gst_video_test_src_smpte (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_smpte75 (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_smpte_rp_219 (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
+{
+  /* heights */
+  int b, b1;
+  int bs[6];                    /* bottom-half 6 rows */
+
+  /* widths */
+  int a, d, k, g, h, m, k1, k2, g1, g2;
+  int cs[7];                    /* 'c's and 'f's */
+  int ij[5];                    /* parts of 'x' and 'y' */
+  int x, y, x1, x2, y1, y2;
+
+  paintinfo pi = PAINT_INFO_INIT;
+  paintinfo *p = &pi;
+
+  a = frame->info.width;
+  b = frame->info.height;
+
+  videotestsrc_setup_paintinfo (v, p, a, b);
+  p->colors = vts_colors_bt709_ycbcr_rp_219;
+
+  /* heights */
+  {
+    /* error distribution for the bottom 6 rows
+     * x.e., pattern 1 is always shorter when b%12 != 0 */
+    static const int b12_err[6][6] = {
+      {0, 0, 0, 0, 0, 0},
+      {0, 1, 0, 0, 0, 0},
+      {0, 1, 1, 0, 0, 0},
+      {0, 0, 0, 1, 1, 1},
+      {0, 1, 1, 1, 0, 1},
+      {0, 1, 1, 1, 1, 1},
+    };
+
+    int b2, b12, b12_r, x;
+
+    b1 = b / 2;
+    b2 = b - b1;
+
+    b12 = b2 / 6;
+    b12_r = b2 % 6;
+    for (x = 0; x < 6; x++) {
+      bs[x] = b12 + b12_err[b12_r][x];
+    }
+  }
+
+  /* widths */
+  {
+    /* error distribution for 'c' columns */
+    static const int c_w_err[7][7] = {
+      {0, 0, 0, 0, 0, 0, 0},
+      {0, 0, 0, 1, 0, 0, 0},
+      {1, 0, 0, 0, 0, 0, 1},
+      {0, 1, 0, 1, 0, 1, 0},
+      {1, 0, 1, 0, 1, 0, 1},
+      {0, 1, 1, 1, 1, 1, 0},
+      {1, 1, 1, 0, 1, 1, 1}
+    };
+
+    /* error distribution for 'x' column in pattern 4 */
+    static const int i_err[3][3] = {
+      {0, 0, 0},
+      {0, 1, 0},
+      {1, 0, 1}
+    };
+
+    int a34, c, cr, kg, dkg, hijm, hijmd, c3, c3r, i;
+
+    d = a / 8;
+    a34 = a - 2 * d;
+
+    /* 'c's and 'f's */
+    c = a34 / 7;
+    cr = a34 % 7;
+    for (i = 0; i < 7; i++) {
+      cs[i] = c + c_w_err[cr][i];
+    }
+
+    dkg = a / 2;
+    hijmd = a - dkg;
+
+    kg = dkg - d;
+    g = cs[0] + cs[1];
+    k = kg - g;
+    k1 = k / 2;
+    k2 = k - k1;
+    g1 = g / 2;
+    g2 = g - g1;
+
+    hijm = hijmd - d;
+
+    /* i = c = 3 * c3 */
+    c3 = cs[5] / 3;
+    c3r = cs[5] % 3;
+    for (i = 0; i < 3; i++) {
+      ij[i] = c3 + i_err[c3r][i];
+    }
+    ij[3] = ij[1];
+    ij[4] = ij[2];
+
+    m = cs[5];                  /* m = c */
+    h = hijm - ij[0] - ij[1] - ij[2] - ij[3] - ij[4] - m;
+  }
+
+  /* pattern 1 */
+  x1 = 0;
+  x2 = d;                       /* 'd' bar size */
+  p->color = p->colors;         /* 40% gray */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[0];              /* 'f' bar size */
+  p->color = p->colors + 1;     /* 75% white */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[1];              /* 'c' bar size */
+  p->color = p->colors + 2;     /* 75% yellow */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[2];              /* 'c' bar size */
+  p->color = p->colors + 3;     /* 75% cyan */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[3];              /* 'c' bar size */
+  p->color = p->colors + 4;     /* 75% green */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[4];              /* 'c' bar size */
+  p->color = p->colors + 5;     /* 75% magenta */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[5];              /* 'c' bar size */
+  p->color = p->colors + 6;     /* 75% red */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[6];              /* 'f' bar size */
+  p->color = p->colors + 7;     /* 75% blue */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + d;                  /* 'd' bar size */
+  p->color = p->colors;         /* 40% gray */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  y1 = 0;
+  y2 = b1 + bs[0];
+  for (y = y1; y < y2; y++)
+    videotestsrc_convert_tmpline (p, frame, y);
+
+  /* pattern 2 */
+  x1 = 0;
+  x2 = d;
+  p->color = p->colors + 8;     /* 100% cyan */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[0];
+  p->color = p->colors + 11;    /* *2: -I, +Q in *3 */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[1] + cs[2] + cs[3] + cs[4] + cs[5] + cs[6];
+  p->color = p->colors + 1;     /* 75% white */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + d;
+  p->color = p->colors + 12;    /* 100% blue */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  y1 = y2;
+  y2 = y1 + bs[1];
+  for (y = y1; y < y2; y++)
+    videotestsrc_convert_tmpline (p, frame, y);
+
+  /* pattern 3 */
+  x1 = 0;
+  x2 = d;
+  p->color = p->colors + 13;    /* 100% yellow */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + cs[0];
+  p->color = p->colors + 15;    /* *3: +Q, -I in *2 */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  /* Y-Ramp, from row 14 to row 9 */
+  x1 = x2;
+  x2 = x1 + cs[1] + cs[2] + cs[3] + cs[4] + cs[5];
+  for (x = x1; x < x2; x++)
+    p->tmpline_u8[x] = 255 * (x - x1) / (x2 - x1);
+  videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8, p->colors + 9,
+      p->colors + 14, x1, x2);
+
+  x1 = x2;
+  x2 = x1 + cs[6];
+  p->color = p->colors + 9;     /* 100% white */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + d;
+  p->color = p->colors + 16;    /* 100% red */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  y1 = y2;
+  y2 = y1 + bs[2];
+  for (y = y1; y < y2; y++)
+    videotestsrc_convert_tmpline (p, frame, y);
+
+  /* pattern 4, subdivision 1 */
+  x1 = 0;
+  x2 = d;
+  p->color = p->colors + 17;    /* *4: 15% gray */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + k;                  /* 3 sub divisions */
+  p->color = p->colors + 14;    /* 0% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + g;                  /* 3 sub divisions */
+  p->color = p->colors + 9;     /* 100% white */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + h;
+  p->color = p->colors + 14;    /* 0% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + ij[0];
+  p->color = p->colors + 20;    /* -2% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + ij[1];
+  p->color = p->colors + 14;    /* 0% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + ij[2];
+  p->color = p->colors + 21;    /* +2% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + ij[3];
+  p->color = p->colors + 20;    /* 0% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + ij[4];
+  p->color = p->colors + 22;    /* +4% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + m;
+  p->color = p->colors + 14;    /* 0% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + d;
+  p->color = p->colors + 17;    /* *4: 15% gray */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  y1 = y2;
+  y2 = y1 + bs[3];
+  for (y = y1; y < y2; y++)
+    videotestsrc_convert_tmpline (p, frame, y);
+
+  /* pattern 4, subdivision 2 */
+
+  /* *5: sub-black valley, from row 14 to row 18 (first half) */
+  x1 = d;
+  x2 = x1 + k1;
+  for (x = x1; x < x2; x++)
+    p->tmpline_u8[x] = 255 * (x - x1) / (x2 - x1);
+  videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8, p->colors + 18,
+      p->colors + 14, x1, x2);
+
+  /* *5: sub-black valley, from row 18 to row 14 (second half) */
+  x1 = x2;
+  x2 = x1 + k2;
+  for (x = x1; x < x2; x++)
+    p->tmpline_u8[x] = 255 * (x - x1) / (x2 - x1);
+  videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8, p->colors + 14,
+      p->colors + 18, x1, x2);
+
+  /* *6: super-white peak, from row 9 to row 19 (first half) */
+  x1 = x2;
+  x2 = x1 + g1;
+  for (x = x1; x < x2; x++)
+    p->tmpline_u8[x] = 255 * (x - x1) / (x2 - x1);
+  videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8, p->colors + 19,
+      p->colors + 9, x1, x2);
+
+  /* *6: super-white peak, from row 19 to row 9 (second half) */
+  x1 = x2;
+  x2 = x1 + g2;
+  for (x = x1; x < x2; x++)
+    p->tmpline_u8[x] = 255 * (x - x1) / (x2 - x1);
+  videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8, p->colors + 9,
+      p->colors + 19, x1, x2);
+
+  y1 = y2;
+  y2 = y1 + bs[4];
+  for (y = y1; y < y2; y++)
+    videotestsrc_convert_tmpline (p, frame, y);
+
+  /* pattern 4, subdivision 3 */
+  x1 = d;
+  x2 = x1 + k;
+  p->color = p->colors + 14;    /* 0% black */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  x1 = x2;
+  x2 = x1 + g;
+  p->color = p->colors + 9;     /* 100% white */
+  p->paint_tmpline (p, x1, (x2 - x1));
+
+  y1 = y2;
+  y2 = y1 + bs[5];
+  for (y = y1; y < y2; y++)
+    videotestsrc_convert_tmpline (p, frame, y);
+}
+
+void
+gst_video_test_src_smpte75 (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -455,7 +840,8 @@ gst_video_test_src_smpte75 (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_smpte100 (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_smpte100 (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -479,7 +865,8 @@ gst_video_test_src_smpte100 (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_bar (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_bar (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int j;
   paintinfo pi = PAINT_INFO_INIT;
@@ -501,7 +888,8 @@ gst_video_test_src_bar (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_snow (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_snow (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -517,11 +905,11 @@ gst_video_test_src_snow (GstVideoTestSrc * v, GstVideoFrame * frame)
 
   for (j = 0; j < h; j++) {
     for (i = 0; i < w; i++) {
-      int y = random_char ();
+      int y = random_char (&v->random_state);
       p->tmpline_u8[i] = y;
     }
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
-        &p->foreground_color, &p->background_color, w);
+        &p->foreground_color, &p->background_color, 0, w);
     videotestsrc_convert_tmpline (p, frame, j);
   }
 }
@@ -552,37 +940,43 @@ gst_video_test_src_unicolor (GstVideoTestSrc * v, GstVideoFrame * frame,
 }
 
 void
-gst_video_test_src_black (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_black (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   gst_video_test_src_unicolor (v, frame, COLOR_BLACK);
 }
 
 void
-gst_video_test_src_white (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_white (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   gst_video_test_src_unicolor (v, frame, COLOR_WHITE);
 }
 
 void
-gst_video_test_src_red (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_red (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   gst_video_test_src_unicolor (v, frame, COLOR_RED);
 }
 
 void
-gst_video_test_src_green (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_green (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   gst_video_test_src_unicolor (v, frame, COLOR_GREEN);
 }
 
 void
-gst_video_test_src_blue (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_blue (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   gst_video_test_src_unicolor (v, frame, COLOR_BLUE);
 }
 
 void
-gst_video_test_src_blink (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_blink (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   paintinfo pi = PAINT_INFO_INIT;
@@ -604,7 +998,8 @@ gst_video_test_src_blink (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_solid (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_solid (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   paintinfo pi = PAINT_INFO_INIT;
@@ -622,7 +1017,8 @@ gst_video_test_src_solid (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_checkers1 (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_checkers1 (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int x, y;
   paintinfo pi = PAINT_INFO_INIT;
@@ -645,7 +1041,8 @@ gst_video_test_src_checkers1 (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_checkers2 (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_checkers2 (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int x, y;
   paintinfo pi = PAINT_INFO_INIT;
@@ -670,7 +1067,8 @@ gst_video_test_src_checkers2 (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_checkers4 (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_checkers4 (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int x, y;
   paintinfo pi = PAINT_INFO_INIT;
@@ -695,7 +1093,8 @@ gst_video_test_src_checkers4 (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_checkers8 (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_checkers8 (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int x, y;
   paintinfo pi = PAINT_INFO_INIT;
@@ -756,7 +1155,8 @@ static const guint8 sine_table[256] = {
 
 
 void
-gst_video_test_src_zoneplate (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_zoneplate (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -865,13 +1265,14 @@ gst_video_test_src_zoneplate (GstVideoTestSrc * v, GstVideoFrame * frame)
       p->tmpline_u8[i] = sine_table[phase & 0xff];
     }
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
-        &p->foreground_color, &p->background_color, w);
+        &p->foreground_color, &p->background_color, 0, w);
     videotestsrc_convert_tmpline (p, frame, j);
   }
 }
 
 void
-gst_video_test_src_chromazoneplate (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_chromazoneplate (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -965,7 +1366,8 @@ gst_video_test_src_chromazoneplate (GstVideoTestSrc * v, GstVideoFrame * frame)
 
 #undef SCALE_AMPLITUDE
 void
-gst_video_test_src_circular (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_circular (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -999,13 +1401,14 @@ gst_video_test_src_circular (GstVideoTestSrc * v, GstVideoFrame * frame)
       }
     }
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
-        &p->foreground_color, &p->background_color, w);
+        &p->foreground_color, &p->background_color, 0, w);
     videotestsrc_convert_tmpline (p, frame, j);
   }
 }
 
 void
-gst_video_test_src_gamut (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_gamut (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int x, y;
   paintinfo pi = PAINT_INFO_INIT;
@@ -1057,27 +1460,79 @@ gst_video_test_src_gamut (GstVideoTestSrc * v, GstVideoFrame * frame)
 }
 
 void
-gst_video_test_src_ball (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_ball (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
-  paintinfo pi = PAINT_INFO_INIT;
-  paintinfo *p = &pi;
-  int t = v->n_frames;
-  double x, y;
   int radius = 20;
   int w = frame->info.width, h = frame->info.height;
+  gint64 wall_time;
+  gdouble rad = 0;
+  double x, y;
+  int flipit = 0;
 
+  paintinfo pi = PAINT_INFO_INIT;
+  paintinfo *p = &pi;
+
+  struct vts_color_struct
+      *foreground_color = &p->foreground_color,
+      *background_color = &p->background_color;
+
+  switch (v->animation_mode) {
+    case GST_VIDEO_TEST_SRC_FRAMES:
+      rad = (gdouble) (v->n_frames) / 200;
+      flipit = (v->n_frames / 50) % 2;
+      break;
+    case GST_VIDEO_TEST_SRC_WALL_TIME:
+      wall_time = g_get_real_time ();
+
+      rad = (gdouble) wall_time / 1000000.0;
+      flipit = (wall_time / 1000000) % 2;
+      break;
+    case GST_VIDEO_TEST_SRC_RUNNING_TIME:
+      rad = (gdouble) (pts) / GST_SECOND;
+      flipit = (pts / GST_SECOND) % 2;
+      break;
+  }
+  if (v->motion_type == GST_VIDEO_TEST_SRC_HSWEEP) {
+    /* Periodic reset for half sweep */
+    rad /= 2;
+    rad -= floor (2 * rad) / 2;
+  }
+
+  /* Scale for the animation calcs */
+  rad = 2 * G_PI * rad;
+
+  if (v->motion_type == GST_VIDEO_TEST_SRC_WAVY) {
+    x = radius + (0.5 + 0.5 * sin (rad)) * (w - 2 * radius);
+    y = radius + (0.5 + 0.5 * sin (rad * sqrt (2))) * (h - 2 * radius);
+  } else {
+    /* sweep and hsweep */
+    /* x,y is center of circle,
+     * radius is radius
+     * rad = angle .. of sweep.
+     */
+
+    radius = MIN (h, w) / 4 - 0;
+
+    /* 0 is the margin between edge of screen and top of ball */
+    x = w / 2 + sin (rad) * radius;
+    y = h / 2 - cos (rad) * radius;
+  }
+
+  if (v->flip && flipit) {
+    foreground_color = &p->background_color;
+    background_color = &p->foreground_color;
+  }
+
+  /* draw ball on frame */
   videotestsrc_setup_paintinfo (v, p, w, h);
-
-  x = radius + (0.5 + 0.5 * sin (2 * G_PI * t / 200)) * (w - 2 * radius);
-  y = radius + (0.5 + 0.5 * sin (2 * G_PI * sqrt (2) * t / 200)) * (h -
-      2 * radius);
-
   for (i = 0; i < h; i++) {
     if (i < y - radius || i > y + radius) {
       memset (p->tmpline_u8, 0, w);
     } else {
-      int r = rint (sqrt (radius * radius - (i - y) * (i - y)));
+      double o = MAX (0, (radius * radius - (i - y) * (i - y)));
+      int r = rint (sqrt (o));
       int x1, x2;
       int j;
 
@@ -1102,10 +1557,30 @@ gst_video_test_src_ball (GstVideoTestSrc * v, GstVideoFrame * frame)
         p->tmpline_u8[j] = 0;
       }
     }
+
+    if ((v->motion_type == GST_VIDEO_TEST_SRC_SWEEP) ||
+        (v->motion_type == GST_VIDEO_TEST_SRC_HSWEEP)) {
+      /* dot in the middle (to draw a line down the center) */
+      p->tmpline_u8[w / 2] = 255;
+      p->tmpline_u8[(int) x] = 255;
+    }
+
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
-        &p->foreground_color, &p->background_color, w);
+        foreground_color, background_color, 0, w);
     videotestsrc_convert_tmpline (p, frame, i);
   }
+
+  if ((v->motion_type == GST_VIDEO_TEST_SRC_SWEEP) ||
+      (v->motion_type == GST_VIDEO_TEST_SRC_HSWEEP)) {
+    /* draw a line across the middle of frame and ball. */
+    for (i = 0; i < w; i++) {
+      p->tmpline_u8[i] = 255;
+    }
+    videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
+        foreground_color, background_color, 0, w);
+    videotestsrc_convert_tmpline (p, frame, h / 2);
+    videotestsrc_convert_tmpline (p, frame, y);
+  }
 }
 
 static void
@@ -1116,9 +1591,9 @@ paint_tmpline_ARGB (paintinfo * p, int x, int w)
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
   value = (p->color->A << 0) | (p->color->R << 8) |
-      (p->color->G << 16) | (p->color->B << 24);
+      (p->color->G << 16) | ((guint32) p->color->B << 24);
 #else
-  value = (p->color->A << 24) | (p->color->R << 16) |
+  value = ((guint32) p->color->A << 24) | (p->color->R << 16) |
       (p->color->G << 8) | (p->color->B << 0);
 #endif
 
@@ -1234,7 +1709,8 @@ convert_hline_bayer (paintinfo * p, GstVideoFrame * frame, int y)
 }
 
 void
-gst_video_test_src_pinwheel (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_pinwheel (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -1278,13 +1754,14 @@ gst_video_test_src_pinwheel (GstVideoTestSrc * v, GstVideoFrame * frame)
       p->tmpline_u8[i] = CLAMP (rint (v * 128 + 128), 0, 255);
     }
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
-        &p->foreground_color, &p->background_color, w);
+        &p->foreground_color, &p->background_color, 0, w);
     videotestsrc_convert_tmpline (p, frame, j);
   }
 }
 
 void
-gst_video_test_src_spokes (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_spokes (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -1329,13 +1806,14 @@ gst_video_test_src_spokes (GstVideoTestSrc * v, GstVideoFrame * frame)
       p->tmpline_u8[i] = CLAMP (rint (v * 255), 0, 255);
     }
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
-        &p->foreground_color, &p->background_color, w);
+        &p->foreground_color, &p->background_color, 0, w);
     videotestsrc_convert_tmpline (p, frame, j);
   }
 }
 
 void
-gst_video_test_src_gradient (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_gradient (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
@@ -1355,13 +1833,14 @@ gst_video_test_src_gradient (GstVideoTestSrc * v, GstVideoFrame * frame)
       p->tmpline_u8[i] = y;
     }
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
-        &p->foreground_color, &p->background_color, w);
+        &p->foreground_color, &p->background_color, 0, w);
     videotestsrc_convert_tmpline (p, frame, j);
   }
 }
 
 void
-gst_video_test_src_colors (GstVideoTestSrc * v, GstVideoFrame * frame)
+gst_video_test_src_colors (GstVideoTestSrc * v, GstClockTime pts,
+    GstVideoFrame * frame)
 {
   int i;
   int j;
