@@ -24,12 +24,16 @@
 #include <locale.h>
 
 #include <gst/gst.h>
-#include <gst/gst-i18n-app.h>
+#include <glib/gi18n.h>
 #include <gst/math-compat.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+#ifdef __APPLE__
+#include <TargetConditionals.h>
+#endif
+
 GST_DEBUG_CATEGORY (devmon_debug);
 #define GST_CAT_DEFAULT devmon_debug
 
@@ -144,6 +148,8 @@ print_structure_field (GQuark field_id, const GValue * value,
   if (G_VALUE_HOLDS_UINT (value)) {
     val = g_strdup_printf ("%u (0x%08x)", g_value_get_uint (value),
         g_value_get_uint (value));
+  } else if (G_VALUE_HOLDS_STRING (value)) {
+    val = g_value_dup_string (value);
   } else {
     val = gst_value_serialize (value);
   }
@@ -280,8 +286,8 @@ quit_loop (GMainLoop * loop)
   return G_SOURCE_REMOVE;
 }
 
-int
-main (int argc, char **argv)
+static int
+real_main (int argc, char **argv)
 {
   gboolean print_version = FALSE;
   GError *err = NULL;
@@ -399,3 +405,13 @@ main (int argc, char **argv)
 
   return 0;
 }
+
+int
+main (int argc, char *argv[])
+{
+#if defined(__APPLE__) && TARGET_OS_MAC && !TARGET_OS_IPHONE
+  return gst_macos_main ((GstMainFunc) real_main, argc, argv, NULL);
+#else
+  return real_main (argc, argv);
+#endif
+}
