@@ -27,7 +27,7 @@
 #include <locale.h>
 
 #include <gst/gst.h>
-#include <gst/gst-i18n-app.h>
+#include <glib/gi18n.h>
 #include <gst/audio/audio.h>
 #include <gst/video/video.h>
 #include <gst/pbutils/pbutils.h>
@@ -42,7 +42,11 @@
 #ifdef HAVE_WINMM
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
-#include <timeapi.h>
+#include <mmsystem.h>
+#endif
+
+#ifdef __APPLE__
+#include <TargetConditionals.h>
 #endif
 
 #include "gst-play-kb.h"
@@ -104,6 +108,7 @@ typedef struct
 
   gboolean buffering;
   gboolean is_live;
+  gboolean initial_file;
 
   GstState desired_state;       /* as per user interaction, PAUSED or PLAYING */
 
@@ -111,6 +116,7 @@ typedef struct
 
   /* configuration */
   gboolean gapless;
+  gboolean instant_uri;
 
   GstPlayTrickMode trick_mode;
   gdouble rate;
@@ -166,8 +172,9 @@ gst_play_printf (const gchar * format, ...)
 
 static GstPlay *
 play_new (gchar ** uris, const gchar * audio_sink, const gchar * video_sink,
-    gboolean gapless, gdouble initial_volume, gboolean verbose,
-    const gchar * flags_string, gboolean use_playbin3, gdouble start_position)
+    gboolean gapless, gboolean instant_uri, gdouble initial_volume,
+    gboolean verbose, const gchar * flags_string, gboolean use_playbin3,
+    gdouble start_position, gboolean no_position)
 {
   GstElement *sink, *playbin;
   GstPlay *play;
@@ -247,8 +254,9 @@ play_new (gchar ** uris, const gchar * audio_sink, const gchar * video_sink,
   play->bus_watch = gst_bus_add_watch (GST_ELEMENT_BUS (play->playbin),
       play_bus_msg, play);
 
-  /* FIXME: make configurable incl. 0 for disable */
-  play->timeout = g_timeout_add (100, play_timeout, play);
+  if (!no_position) {
+    play->timeout = g_timeout_add (100, play_timeout, play);
+  }
 
   play->missing = NULL;
   play->buffering = FALSE;
@@ -262,6 +270,11 @@ play_new (gchar ** uris, const gchar * audio_sink, const gchar * video_sink,
         G_CALLBACK (play_about_to_finish), play);
   }
 
+  play->initial_file = TRUE;
+  if (use_playbin3) {
+    play->instant_uri = instant_uri;
+    g_object_set (G_OBJECT (play->playbin), "instant-uri", instant_uri, NULL);
+  }
   if (initial_volume != -1)
     play_set_relative_volume (play, initial_volume - 1.0);
 
@@ -284,7 +297,8 @@ play_free (GstPlay * play)
   gst_object_unref (play->playbin);
 
   g_source_remove (play->bus_watch);
-  g_source_remove (play->timeout);
+  if (play->timeout != 0)
+    g_source_remove (play->timeout);
   g_main_loop_unref (play->loop);
 
   g_strfreev (play->uris);
@@ -501,6 +515,8 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
             case GST_NAVIGATION_EVENT_KEY_PRESS:
             {
               const gchar *key;
+              const gchar *key_input;
+              gchar key_adjusted[2];
 
               if (gst_navigation_event_parse_key_event (ev, &key)) {
                 GST_INFO ("Key press: %s", key);
@@ -529,7 +545,21 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
                   break;
                 }
 
-                keyboard_cb (key, user_data);
+                /* In the case of a simple single-char input,
+                 * make it lower or upper as needed, and
+                 * send that instead */
+                if (key[0] != '\0' && key[1] == '\0') {
+                  if (play->shift_pressed)
+                    key_adjusted[0] = g_ascii_toupper (key[0]);
+                  else
+                    key_adjusted[0] = g_ascii_tolower (key[0]);
+                  key_adjusted[1] = '\0';
+                  key_input = key_adjusted;
+                } else {
+                  key_input = key;
+                }
+
+                keyboard_cb (key_input, user_data);
               }
               break;
             }
@@ -604,8 +634,9 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
 
       if (collection) {
         g_mutex_lock (&play->selection_lock);
-        gst_object_replace ((GstObject **) & play->collection,
-            (GstObject *) collection);
+        if (play->collection)
+          gst_object_unref (play->collection);
+        play->collection = collection;
         g_mutex_unlock (&play->selection_lock);
       }
       break;
@@ -643,7 +674,7 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
             } else if (type & GST_STREAM_TYPE_TEXT) {
               play->cur_text_sid = g_strdup (stream_id);
             } else {
-              gst_print ("Unknown stream type with stream-id %s", stream_id);
+              gst_print ("Unknown stream type with stream-id %s\n", stream_id);
             }
             gst_object_unref (stream);
           }
@@ -692,7 +723,7 @@ play_timeout (gpointer user_data)
     status[len] = '\0';
   }
 
-  if (pos >= 0 && dur > 0) {
+  if (pos >= 0) {
     gchar dstr[32], pstr[32];
 
     /* FIXME: pretty print in nicer format */
@@ -728,7 +759,8 @@ play_uri (GstPlay * play, const gchar * next_uri)
 {
   gchar *loc;
 
-  gst_element_set_state (play->playbin, GST_STATE_READY);
+  if (!play->instant_uri || play->initial_file)
+    gst_element_set_state (play->playbin, GST_STATE_READY);
   play_reset (play);
 
   loc = play_uri_get_display_name (play, next_uri);
@@ -737,23 +769,26 @@ play_uri (GstPlay * play, const gchar * next_uri)
 
   g_object_set (play->playbin, "uri", next_uri, NULL);
 
-  switch (gst_element_set_state (play->playbin, GST_STATE_PAUSED)) {
-    case GST_STATE_CHANGE_FAILURE:
-      /* ignore, we should get an error message posted on the bus */
-      break;
-    case GST_STATE_CHANGE_NO_PREROLL:
-      gst_print ("Pipeline is live.\n");
-      play->is_live = TRUE;
-      break;
-    case GST_STATE_CHANGE_ASYNC:
-      gst_print ("Prerolling...\r");
-      break;
-    default:
-      break;
-  }
+  if (!play->instant_uri || play->initial_file) {
+    switch (gst_element_set_state (play->playbin, GST_STATE_PAUSED)) {
+      case GST_STATE_CHANGE_FAILURE:
+        /* ignore, we should get an error message posted on the bus */
+        break;
+      case GST_STATE_CHANGE_NO_PREROLL:
+        gst_print ("Pipeline is live.\n");
+        play->is_live = TRUE;
+        break;
+      case GST_STATE_CHANGE_ASYNC:
+        gst_print ("Prerolling...\r");
+        break;
+      default:
+        break;
+    }
 
-  if (play->desired_state != GST_STATE_PAUSED)
-    gst_element_set_state (play->playbin, play->desired_state);
+    if (play->desired_state != GST_STATE_PAUSED)
+      gst_element_set_state (play->playbin, play->desired_state);
+  }
+  play->initial_file = FALSE;
 }
 
 /* returns FALSE if we have reached the end of the playlist */
@@ -1158,7 +1193,8 @@ play_get_nth_stream_in_collection (GstPlay * play, guint index,
 }
 
 static void
-play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
+play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type,
+    gboolean forward)
 {
   const gchar *prop_cur, *prop_n, *prop_get, *name;
   gint cur = -1, n = -1;
@@ -1270,19 +1306,34 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
 
   if (play->is_playbin3) {
     if (n > 0) {
-      if (cur < 0)
-        cur = 0;
-      else
-        cur = (cur + 1) % (n + 1);
+      if (forward) {
+        if (cur < 0)
+          cur = 0;
+        else
+          cur = (cur + 1) % (n + 1);
+      } else {
+        if (cur <= 0)
+          cur = n;
+        else
+          cur = (cur - 1) % (n + 1);
+      }
     }
   } else {
     g_object_get (play->playbin, prop_cur, &cur, prop_n, &n, "flags",
         &cur_flags, NULL);
 
-    if (!(cur_flags & flag))
-      cur = 0;
-    else
-      cur = (cur + 1) % (n + 1);
+    if (forward) {
+      if (!(cur_flags & flag))
+        cur = 0;
+      else
+        cur = (cur + 1) % (n + 1);
+
+    } else {
+      if (cur <= 0)
+        cur = n;
+      else
+        cur = (cur - 1) % (n + 1);
+    }
   }
 
   if (n < 1) {
@@ -1382,9 +1433,9 @@ print_keyboard_help (void)
     "-", N_("decrease playback rate")}, {
     "d", N_("change playback direction")}, {
     "t", N_("enable/disable trick modes")}, {
-    "a", N_("change audio track")}, {
-    "v", N_("change video track")}, {
-    "s", N_("change subtitle track")}, {
+    "A/a", N_("change to previous/next audio track")}, {
+    "V/v", N_("change to previous/next video track")}, {
+    "S/s", N_("change to previous/next subtitle track")}, {
     "0", N_("seek to beginning")}, {
   "k", N_("show keyboard shortcuts")},};
   /* *INDENT-ON* */
@@ -1413,9 +1464,12 @@ keyboard_cb (const gchar * key_input, gpointer user_data)
   GstPlay *play = (GstPlay *) user_data;
   gchar key = '\0';
 
-  /* only want to switch/case on single char, not first char of string */
-  if (key_input[0] != '\0' && key_input[1] == '\0')
-    key = g_ascii_tolower (key_input[0]);
+  /* Switch on the first char for single char inputs,
+   * otherwise leave key = '\0' to fall through to
+   * the default case below */
+  if (key_input[0] != '\0' && key_input[1] == '\0') {
+    key = key_input[0];
+  }
 
   switch (key) {
     case 'k':
@@ -1471,13 +1525,17 @@ keyboard_cb (const gchar * key_input, gpointer user_data)
         break;
       }
     case 'a':
-      play_cycle_track_selection (play, GST_PLAY_TRACK_TYPE_AUDIO);
+    case 'A':
+      play_cycle_track_selection (play, GST_PLAY_TRACK_TYPE_AUDIO, key == 'a');
       break;
     case 'v':
-      play_cycle_track_selection (play, GST_PLAY_TRACK_TYPE_VIDEO);
+    case 'V':
+      play_cycle_track_selection (play, GST_PLAY_TRACK_TYPE_VIDEO, key == 'v');
       break;
     case 's':
-      play_cycle_track_selection (play, GST_PLAY_TRACK_TYPE_SUBTITLE);
+    case 'S':
+      play_cycle_track_selection (play, GST_PLAY_TRACK_TYPE_SUBTITLE,
+          key == 's');
       break;
     case '0':
       play_do_seek (play, 0, play->rate, play->trick_mode);
@@ -1540,8 +1598,8 @@ clear_winmm_timer_resolution (guint resolution)
 }
 #endif
 
-int
-main (int argc, char **argv)
+static int
+real_main (int argc, char **argv)
 {
   GstPlay *play;
   GPtrArray *playlist;
@@ -1549,6 +1607,7 @@ main (int argc, char **argv)
   gboolean print_version = FALSE;
   gboolean interactive = TRUE;
   gboolean gapless = FALSE;
+  gboolean instant_uri = FALSE;
   gboolean shuffle = FALSE;
   gdouble volume = -1;
   gdouble start_position = 0;
@@ -1562,6 +1621,7 @@ main (int argc, char **argv)
   GOptionContext *ctx;
   gchar *playlist_file = NULL;
   gboolean use_playbin3 = FALSE;
+  gboolean no_position = FALSE;
 #ifdef HAVE_WINMM
   guint winmm_timer_resolution = 0;
 #endif
@@ -1579,6 +1639,8 @@ main (int argc, char **argv)
         N_("Audio sink to use (default is autoaudiosink)"), NULL},
     {"gapless", 0, 0, G_OPTION_ARG_NONE, &gapless,
         N_("Enable gapless playback"), NULL},
+    {"instant-uri", 0, 0, G_OPTION_ARG_NONE, &instant_uri,
+        N_("Enable instantaneous uri changes (only with playbin3)"), NULL},
     {"shuffle", 0, 0, G_OPTION_ARG_NONE, &shuffle,
         N_("Shuffle playlist"), NULL},
     {"no-interactive", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,
@@ -1597,14 +1659,17 @@ main (int argc, char **argv)
     {"quiet", 'q', 0, G_OPTION_ARG_NONE, &quiet,
         N_("Do not print any output (apart from errors)"), NULL},
     {"use-playbin3", 0, 0, G_OPTION_ARG_NONE, &use_playbin3,
-          N_("Use playbin3 pipeline")
-          N_("(default varies depending on 'USE_PLAYBIN' env variable)"),
+          N_("Use playbin3 pipeline "
+              "(default varies depending on 'USE_PLAYBIN' env variable)"),
         NULL},
     {"wait-on-eos", 0, 0, G_OPTION_ARG_NONE, &wait_on_eos,
           N_
           ("Keep showing the last frame on EOS until quit or playlist change command "
               "(gapless is ignored)"),
         NULL},
+    {"no-position", 0, 0, G_OPTION_ARG_NONE, &no_position,
+          N_("Do not print current position of pipeline"),
+        NULL},
     {G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames, NULL},
     {NULL}
   };
@@ -1620,6 +1685,7 @@ main (int argc, char **argv)
   g_set_prgname ("gst-play-" GST_API_VERSION);
   /* Ensure XInitThreads() is called if/when needed */
   g_setenv ("GST_GL_XINITTHREADS", "1", TRUE);
+  g_setenv ("GST_XINITTHREADS", "1", TRUE);
 
   ctx = g_option_context_new ("FILE1|URI1 [FILE2|URI2] [FILE3|URI3] ...");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
@@ -1713,8 +1779,9 @@ main (int argc, char **argv)
     shuffle_uris (uris, num);
 
   /* prepare */
-  play = play_new (uris, audio_sink, video_sink, gapless, volume, verbose,
-      flags, use_playbin3, start_position);
+  play =
+      play_new (uris, audio_sink, video_sink, gapless, instant_uri, volume,
+      verbose, flags, use_playbin3, start_position, no_position);
 
   if (play == NULL) {
     gst_printerr
@@ -1765,3 +1832,13 @@ main (int argc, char **argv)
   gst_deinit ();
   return 0;
 }
+
+int
+main (int argc, char *argv[])
+{
+#if defined(__APPLE__) && TARGET_OS_MAC && !TARGET_OS_IPHONE
+  return gst_macos_main ((GstMainFunc) real_main, argc, argv, NULL);
+#else
+  return real_main (argc, argv);
+#endif
+}
